/*----------------------------------------------------------------------------
 *  netcalc.c - calculate network ip adresses
 *
 *  usage:
 *
 *    print broadcast address:
 *	netcalc broadcast NETWORK:NETMASK
 *	netcalc broadcast NETWORK NETMASK
 *	netcalc broadcast NETWORK:NETMASKBITS
 *	netcalc broadcast NETWORK NETMASKBITS
 *
 *    print netmask:
 *	netcalc netmask NETWORK:NETMASK
 *	netcalc netmask NETWORK/NETMASKBITS
 *	netcalc netmask NETMASKBITS
 *
 *    print network address:
 *	netcalc network IPADDR:NETMASK
 *	netcalc network IPADDR NETMASK
 *	netcalc network IPADDR/NETMASKBITS
 *	netcalc network IPADDR NETMASKBITS
 *
 *    print dns network:
 *	netcalc dnsnet IPADDR:NETMASK
 *	netcalc dnsnet IPADDR NETMASK
 *	netcalc dnsnet IPADDR/NETMASKBITS
 *	netcalc dnsnet IPADDR NETMASKBITS
 *
 *    print reverse dns network:
 *	netcalc dnsrev IPADDR:NETMASK
 *	netcalc dnsrev IPADDR NETMASK
 *	netcalc dnsrev IPADDR/NETMASKBITS
 *	netcalc dnsrev IPADDR NETMASKBITS
 *
 *  Copyright (c) 2000-2001 Frank Meyer <frank@fli4l.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Creation:	    07.11.2000  fm
 *  Last Update:    09.09.2001  fm
 *----------------------------------------------------------------------------
 */

#include <stdio.h>
#include <string.h>

#define TRUE	1
#define FALSE	0

static int	broadcast (int, char **);
static int	netmask (int, char **);
static int	network (int, char **);
static int	dnsnet (int, char **, int);
static int	is_numeric (char *);
static void	set_netmask (int *, int);
static void	usage (char *);

int
main (int argc, char ** argv)
{
    if (argc < 3 || argc > 5)
    {
	usage (argv[0]);
    }

    if (! strcmp (argv[1], "broadcast"))
    {
	return (broadcast (argc, argv));
    }

    if (! strcmp (argv[1], "netmask"))
    {
	return (netmask (argc, argv));
    }

    if (! strcmp (argv[1], "network"))
    {
	return (network (argc, argv));
    }

    if (! strcmp (argv[1], "dnsnet"))
    {
	return (dnsnet (argc, argv, FALSE));
    }

    if (! strcmp (argv[1], "dnsrev"))
    {
	return (dnsnet (argc, argv, TRUE));
    }

    usage (argv[0]);
    return (0);						/* keep cc happy    */
} /* main (argc, argv) */

/*----------------------------------------------------------------------------
 *  broadcast (argc, argv)
 *
 *  netcalc broadcast 192.168.1.0:255.255.255.0
 *  netcalc broadcast 192.168.1.0 255.255.255.0
 *
 *  netcalc broadcast 192.168.1.0/24
 *  netcalc broadcast 192.168.1.0 24
 *----------------------------------------------------------------------------
 */

static int
broadcast (int argc, char ** argv)
{
    unsigned int    network[4];				/* network address  */
    unsigned int    netmask[4];				/* netmask	    */
    unsigned int    broadcast[4];			/* broadcast	    */
    char *	    netwp;
    char *	    netmp;
    int		    i;

    netwp = argv[2];

    if (argc == 3)
    {
	netmp = strchr (netwp, ':');

	if (! netmp)
	{
	    netmp = strchr (netwp, '/');

	    if (! netmp)
	    {
		usage (argv[0]);
	    }
	}

	*netmp++ = '\0';
    }
    else
    {
	netmp = argv[3];
    }

    if (sscanf (netwp, "%d.%d.%d.%d", &(network[0]), &(network[1]),
					&(network[2]), &(network[3])) != 4)
    {
	fprintf (stderr, "%s: network: invalid address: %s\n", argv[0],
		 netwp);
	exit (1);
    }

    if (sscanf (netmp, "%d.%d.%d.%d", &(netmask[0]), &(netmask[1]),
					&(netmask[2]), &(netmask[3])) != 4)
    {
	int n_bits;

	if (! is_numeric (netmp) ||
	    sscanf (netmp, "%d", &n_bits) != 1 ||
	    (n_bits < 2 || n_bits > 30))
	{
	    fprintf (stderr, "%s: netmask bits: invalid value: %s\n", argv[0],
		     netmp);
	    exit (1);
	}

	set_netmask (netmask, n_bits);
    }

    for (i = 0; i < 4; i++)
    {
	broadcast[i] = network[i] | ((netmask[i] ^ 0xff));
    }

    printf ("%d.%d.%d.%d\n", broadcast[0], broadcast[1],
			     broadcast[2], broadcast[3]);
    return (0);
} /* broadcast (argc, argv) */


/*----------------------------------------------------------------------------
 *  netmask (argc, argv)
 *
 *  netcalc netmask 192.168.1.0:255.255.255.0
 *  netcalc netmask 192.168.1.0/24
 *  netcalc netmask 24
 *----------------------------------------------------------------------------
 */

static int
netmask (int argc, char ** argv)
{
    unsigned int    netmask[4];				/* netmask	    */
    char *	    netwp;
    char *	    netmp;

    netwp = argv[2];

    if (argc == 4)
    {
	usage (argv[0]);
    }

    netmp = strchr (netwp, ':');

    if (netmp)
    {
	netmp++;
    }
    else
    {
	netmp = strchr (netwp, '/');

	if (! netmp)
	{
	    netmp = netwp;
	}
	else
	{
	    netmp++;
	}
    }

    if (sscanf (netmp, "%d.%d.%d.%d", &(netmask[0]), &(netmask[1]),
					&(netmask[2]), &(netmask[3])) != 4)
    {
	int n_bits;

	if (! is_numeric (netmp) ||
	    sscanf (netmp, "%d", &n_bits) != 1 ||
	    (n_bits < 2 || n_bits > 30))
	{
	    fprintf (stderr, "%s: netmask bits: invalid value: %s\n", argv[0],
		     netmp);
	    exit (1);
	}

	set_netmask (netmask, n_bits);
    }

    printf ("%d.%d.%d.%d\n", netmask[0], netmask[1],
			     netmask[2], netmask[3]);
    return (0);
} /* netmask (argc, argv) */


/*----------------------------------------------------------------------------
 *  network (argc, argv)
 *
 *  netcalc network 192.168.1.2:255.255.255.0
 *  netcalc network 192.168.1.2 255.255.255.0
 *
 *  netcalc network 192.168.1.2/24
 *  netcalc network 192.168.1.2 24
 *----------------------------------------------------------------------------
 */

static int
network (int argc, char ** argv)
{
    unsigned int    network[4];				/* network address  */
    unsigned int    netmask[4];				/* netmask	    */
    char *	    ipp;
    char *	    netmp;
    int		    i;

    ipp = argv[2];

    if (argc == 3)
    {
	netmp = strchr (ipp, ':');

	if (! netmp)
	{
	    netmp = strchr (ipp, '/');

	    if (! netmp)
	    {
		usage (argv[0]);
	    }
	}

	*netmp++ = '\0';
    }
    else
    {
	netmp = argv[3];
    }

    if (sscanf (ipp, "%d.%d.%d.%d", &(network[0]), &(network[1]),
					&(network[2]), &(network[3])) != 4)
    {
	fprintf (stderr, "%s: network: invalid address: %s\n", argv[0],
		 ipp);
	exit (1);
    }

    if (sscanf (netmp, "%d.%d.%d.%d", &(netmask[0]), &(netmask[1]),
					&(netmask[2]), &(netmask[3])) != 4)
    {
	int n_bits;

	if (! is_numeric (netmp) ||
	    sscanf (netmp, "%d", &n_bits) != 1 ||
	    (n_bits < 2 || n_bits > 30))
	{
	    fprintf (stderr, "%s: netmask bits: invalid value: %s\n", argv[0],
		     netmp);
	    exit (1);
	}

	set_netmask (netmask, n_bits);
    }

    for (i = 0; i < 4; i++)
    {
	network[i] &= netmask[i];
    }

    printf ("%d.%d.%d.%d\n", network[0], network[1],
			     network[2], network[3]);
    return (0);
} /* network (argc, argv) */

/*----------------------------------------------------------------------------
 *  dnsnet (argc, argv, reverse)
 *
 *  netcalc dnsnet 192.168.1.2:255.255.255.0
 *  netcalc dnsnet 192.168.1.2 255.255.255.0
 *
 *  netcalc dnsnet 192.168.1.2/24
 *  netcalc dnsnet 192.168.1.2 24
 *
 *  netcalc dnsrev 192.168.1.2:255.255.255.0
 *  netcalc dnsrev 192.168.1.2 255.255.255.0
 *
 *  netcalc dnsrev 192.168.1.2/24
 *  netcalc dnsrev 192.168.1.2 24
 *----------------------------------------------------------------------------
 */

static int
dnsnet (int argc, char ** argv, int reverse)
{
    unsigned int    network[4];				/* network address  */
    unsigned int    netmask[4];				/* netmask	    */
    char *	    ipp;
    char *	    netmp;
    int		    i;

    ipp = argv[2];

    if (argc == 3)
    {
	netmp = strchr (ipp, ':');

	if (! netmp)
	{
	    netmp = strchr (ipp, '/');

	    if (! netmp)
	    {
		usage (argv[0]);
	    }
	}

	*netmp++ = '\0';
    }
    else
    {
	netmp = argv[3];
    }

    if (sscanf (ipp, "%d.%d.%d.%d", &(network[0]), &(network[1]),
					&(network[2]), &(network[3])) != 4)
    {
	fprintf (stderr, "%s: network: invalid address: %s\n", argv[0],
		 ipp);
	exit (1);
    }

    if (sscanf (netmp, "%d.%d.%d.%d", &(netmask[0]), &(netmask[1]),
					&(netmask[2]), &(netmask[3])) != 4)
    {
	int n_bits;

	if (! is_numeric (netmp) ||
	    sscanf (netmp, "%d", &n_bits) != 1 ||
	    (n_bits < 2 || n_bits > 30))
	{
	    fprintf (stderr, "%s: netmask bits: invalid value: %s\n", argv[0],
		     netmp);
	    exit (1);
	}

	set_netmask (netmask, n_bits);
    }

    for (i = 0; i < 4; i++)
    {
	if (netmask[i] != 255)
	{
	    netmask[i] = 0;
	}
	network[i] &= netmask[i];
    }

    if (reverse)
    {
	if (netmask[3] != 0)
	{
	    printf ("%d.%d.%d.%d.in-addr.arpa\n", network[3], network[2],
				     network[1], network[0]);
	}
	else if (netmask[2] != 0)
	{
	    printf ("%d.%d.%d.in-addr.arpa\n", network[2], network[1],
		    network[0]);
	}
	else if (netmask[1] != 0)
	{
	    printf ("%d.%d.in-addr.arpa\n", network[1], network[0]);
	}
	else
	{
	    printf ("%d.in-addr.arpa\n", network[0]);
	}
    }
    else
    {
	if (netmask[3] != 0)
	{
	    printf ("%d.%d.%d.%d\n", network[0], network[1],
				     network[2], network[3]);
	}
	else if (netmask[2] != 0)
	{
	    printf ("%d.%d.%d\n", network[0], network[1], network[2]);
	}
	else if (netmask[1] != 0)
	{
	    printf ("%d.%d\n", network[0], network[1]);
	}
	else
	{
	    printf ("%d\n", network[0]);
	}
    }

    return (0);
} /* dnsnet (argc, argv) */


/*----------------------------------------------------------------------------
 *  is_numeric (str)
 *----------------------------------------------------------------------------
 */

static int
is_numeric (char * str)
{
    while (*str)
    {
	if (*str < '0' || *str > '9')			    /* just for fun */
	{
	    return (FALSE);
	}

	str++;
    }
    return (TRUE);
} /* is_numeric (str) */

/*----------------------------------------------------------------------------
 *  set_netmask (netmask, n_bits)
 *----------------------------------------------------------------------------
 */

static void
set_netmask (int * netmask, int n_bits)
{
    int	i;
    int	b;
    int	idx;

    for (i = 0; i < 4; i++)
    {
	netmask[i] = 0;
    }

    for (b = 0; b < n_bits; b++)
    {
	idx = b / 8;

	i = (31 - b) % 8;

	netmask[idx] |= (1 << i);
    }

    return;
} /* set_netmask (netmask, n_bits) */


/*----------------------------------------------------------------------------
 *  usage (pgm_name);
 *----------------------------------------------------------------------------
 */

static void
usage (char * pgm_name)
{
    fprintf (stderr, "usage:\n");
    putc ('\n', stderr);
    fprintf (stderr, "  print broadcast address:\n");
    fprintf (stderr, "    %s broadcast NETWORK:NETMASK\n", pgm_name);
    fprintf (stderr, "    %s broadcast NETWORK NETMASK\n", pgm_name);
    fprintf (stderr, "    %s broadcast NETWORK:NETMASKBITS\n", pgm_name);
    fprintf (stderr, "    %s broadcast NETWORK NETMASKBITS\n", pgm_name);
    putc ('\n', stderr);
    fprintf (stderr, "  print netmask:\n");
    fprintf (stderr, "    %s netmask NETWORK:NETMASK\n", pgm_name);
    fprintf (stderr, "    %s netmask NETWORK/NETMASKBITS\n", pgm_name);
    fprintf (stderr, "    %s netmask NETMASKBITS\n", pgm_name);
    putc ('\n', stderr);
    fprintf (stderr, "  print network address:\n");
    fprintf (stderr, "    %s network IPADDR:NETMASK\n", pgm_name);
    fprintf (stderr, "    %s network IPADDR NETMASK\n", pgm_name);
    fprintf (stderr, "    %s network IPADDR/NETMASKBITS\n", pgm_name);
    fprintf (stderr, "    %s network IPADDR NETMASKBITS\n", pgm_name);
    putc ('\n', stderr);
    fprintf (stderr, "  print dnscache network address:\n");
    fprintf (stderr, "    %s dnsnet|dnsrev IPADDR:NETMASK\n", pgm_name);
    fprintf (stderr, "    %s dnsnet|dnsrev IPADDR NETMASK\n", pgm_name);
    fprintf (stderr, "    %s dnsnet|dnsrev IPADDR/NETMASKBITS\n", pgm_name);
    fprintf (stderr, "    %s dnsnet|dnsrev IPADDR NETMASKBITS\n", pgm_name);

    exit (1);
} /* usage (pgm_name) */
