/*----------------------------------------------------------------------------
 *  check.c   - check variables
 *
 *  Copyright (c) 2001 Frank Meyer
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Creation:       12.08.2001  fm
 *  Last Update:    12.08.2001  fm
 *----------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>
#include "check.h"
#include "var.h"

#ifndef TRUE
# define TRUE               1
# define FALSE              0
#endif

#define OK		    0
#define ERR		    (-1)

#define SUNDAY		    0			    /* 1st day of unix week */
#define HOURS_PER_DAY	    24			    /* hours per day	    */
#define DAYS_PER_WEEK	    7			    /* days per week	    */

static int	time_table[HOURS_PER_DAY * DAYS_PER_WEEK];  /* 24 * 7 hours */
static int	local_time_table[HOURS_PER_DAY * DAYS_PER_WEEK];
static char **	time_variables;
static char **	time_values;

static char *	en_wday_strings[DAYS_PER_WEEK] =
{
    "Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"
};

static char *	ge_wday_strings[DAYS_PER_WEEK] =
{
    "So", "Mo", "Di", "Mi", "Do", "Fr", "Sa"
};

typedef struct check
{
    char *  name;
    char *  opt_var;
    char *  var_n;
    int	    value;
} CHECK;

static CHECK		c_variables[MAX_VARIABLES];
static int		c_variables_used = 0;

extern FILE *		logfp;
extern void		log_perror (char *);
static int 		check_local_time_table (int times_idx);
static void		init_local_time_table (void);

int
read_check_file (char * fname)
{
    char        buf[256];
    char *	p;
    FILE *      fp;
    char	name[64];
    char	opt_var[64];
    char	var_n[64];
    char	value[64];
    int         line;

    fp = fopen (fname, "r");

    if (! fp)
    {
	log_perror (fname);
	return (ERR);
    }

    for (line = 1; fgets (buf, 256, fp); line++)
    {
	if (buf[0] == '#' && buf[1] == '#')	/* double comment: '##'	    */
	{					/* don't copy it!	    */
	    continue;
	}

	p = strchr (buf, '\r');			/* unix: we read a DOS file */

	if (p)
	{
	    *p = '\0';
	}
	else
	{
	    p = strchr (buf, '\n');

	    if (p)
	    {
		*p = '\0';
	    }
	}

	for (p = buf; *p; p++)
	{
	    if (*p != ' ' && *p != '\t')
	    {
		break;
	    }
	}

	if (*p != '\0' && *p != '#')
	{
	    if (sscanf (buf, "%s %s %s %s",
		name, opt_var, var_n, value) != 4)
	    {
		fprintf (stderr, "%s: syntax error in line %d\n",
			 fname, line);
		fprintf (logfp, "%s: syntax error in line %d\n",
			 fname, line);
		fclose (fp);
		return (ERR);
	    }

	    if (c_variables_used < MAX_VARIABLES - 1)
	    {
		c_variables[c_variables_used].name	    = strsave (name);

		if (! strcmp (opt_var, "-"))
		{
		    c_variables[c_variables_used].opt_var   = (char *) NULL;
		}
		else
		{
		    c_variables[c_variables_used].opt_var   = strsave (opt_var);
		}

		if (! strcmp (var_n, "-"))
		{
		    c_variables[c_variables_used].var_n = (char *) NULL;
		}
		else
		{
		    c_variables[c_variables_used].var_n = strsave (var_n);
		}

		if (! strcmp (value, "NONE"))
		{
		    c_variables[c_variables_used].value = CHECK_NONE;
		}
		else if (! strcmp (value, "NOTEMPTY"))
		{
		    c_variables[c_variables_used].value = CHECK_NOTEMPTY;
		}
		else if (! strcmp (value, "YESNO"))
		{
		    c_variables[c_variables_used].value = CHECK_YESNO;
		}
		else if (! strcmp (value, "NUMERIC"))
		{
		    c_variables[c_variables_used].value = CHECK_NUMERIC;
		}
		else if (! strcmp (value, "IPADDR"))
		{
		    c_variables[c_variables_used].value = CHECK_IPADDR;
		}
		else if (! strcmp (value, "EIPADDR"))
		{
		    c_variables[c_variables_used].value = CHECK_EIPADDR;
		}
		else if (! strcmp (value, "NOBLANK"))
		{
		    c_variables[c_variables_used].value = CHECK_NOBLANK;
		}
		else if (! strcmp (value, "DIALMODE"))
		{
		    c_variables[c_variables_used].value = CHECK_DIALMODE;
		}
		else
		{
		    fprintf (stderr, "%s: line %d: unknown value '%s'\n",
			     fname, line, value);
		    fprintf (logfp, "%s: line %d: unknown value '%s'\n",
			     fname, line, value);
		    fclose (fp);
		    return (ERR);
		}
	    }
	    else
	    {
		fprintf (stderr, "too many variables, max is %d.\n",
			 MAX_VARIABLES);
		fprintf (logfp, "too many variables, max is %d.\n",
			 MAX_VARIABLES);
		fclose (fp);
		return (ERR);
	    }

	    c_variables_used++;
	}

	line++;
    }

    fclose (fp);

    return (OK);
} /* read_check_file (char * fname) */

int
get_variable_dimension (char * variable)
{
    int	    index_found;
    int	    i;

    index_found = FALSE;

    if (index_found)
    {
	for (i = 0; i < c_variables_used; i++)
	{
	    if (! strcmp (c_variables[i].name, variable))
	    {				/* vbox variables are no arrays!    */
		return (ERR);		/* --> no real dimension	    */
	    }

	    if (! strcmp (c_variables[i].name, variable))
	    {
		if (c_variables[i].var_n)
		{
		    char *  value;
		    int	    n;

		    value = get_variable (c_variables[i].var_n);

		    if (! value)
		    {
			fprintf (stderr, "variable '%s' not defined\n",
				 c_variables[i].var_n);
			fprintf (logfp, "variable '%s' not defined\n",
				 c_variables[i].var_n);
			return (FATAL_ERR);
		    }

		    n = atoi (value);

		    return (n);
		}
		else
		{
		    return (ERR);		    /* no dimension!	    */
		}
	    }
	}

	fprintf (stderr, "variable '%s' not found in check file\n", variable);
	fprintf (logfp, "variable '%s' not found in check file\n", variable);
	return (FATAL_ERR);			    /* variable not found   */
    }

    return (ERR);
} /* get_variable_dimension (char * variable) */

char *
get_var_n (char * variable)
{
    char    buf[64];
    char *  varp;
    int	    index_found;
    int	    i;

    index_found = FALSE;
    varp = variable;

    for (i = 0; i < 64 - 1 && *varp; i++)
    {
	if (*varp >= '0' && *varp <= '9')
	{
	    buf[i] = '%';
	    index_found = TRUE;

	    while (*varp >= '0' && *varp <= '9')
	    {
		varp++;
	    }
	}
	else
	{
	    buf[i] = *varp++;
	}
    }

    buf[i] = '\0';

    if (index_found)
    {
	for (i = 0; i < c_variables_used; i++)
	{
	    if (! strcmp (c_variables[i].name, variable))
	    {				/* vbox variables are no arrays!    */
		return (NULL);		/* --> no real dimension	    */
	    }

	    if (! strcmp (c_variables[i].name, buf))
	    {
		return (c_variables[i].var_n);
	    }
	}

	fprintf (stderr, "variable '%s' not found in check file\n", buf);
	fprintf (logfp, "variable '%s' not found in check file\n", buf);
    }

    return ((char *) NULL);
}

/*----------------------------------------------------------------------------
 * convert_week_day_to_day ()		    - convert week day to index 0 - 6
 *----------------------------------------------------------------------------
 */
static int
convert_week_day_to_day (char * week_day)
{
    int	i;

    for (i = 0; i < DAYS_PER_WEEK; i++)
    {
	if (! strcmp (week_day, en_wday_strings[i]) ||
	    ! strcmp (week_day, ge_wday_strings[i]))
	{
	    return (i);
	}
    }

    return (-1);
} /* convert_week_day_to_day (week_day) */


/*----------------------------------------------------------------------------
 * init_local_time_table ()		    - initialize local time table
 *----------------------------------------------------------------------------
 */
void
init_local_time_table (void)
{
    int i;
    for (i=0; i< DAYS_PER_WEEK*HOURS_PER_DAY; i++)
    {
	local_time_table [i] = -1;
    }
/*     memset (local_time_table, -1, HOURS_PER_DAY * DAYS_PER_WEEK * sizeof (int)); */
}
/*----------------------------------------------------------------------------
 * check_local_time_table () - check whether time table spans the whole week
 *----------------------------------------------------------------------------
 */
int
check_local_time_table (int times_idx)
{
    int day, hour;
    int start_day, start_hour;
    int found = 0;
    int ret = OK;
    for (day = 0; day < DAYS_PER_WEEK; day++)
    {
	for (hour = 0; hour < HOURS_PER_DAY; hour++)
	{
	    if (!found)
	    {
		if (local_time_table [day*HOURS_PER_DAY + hour] == -1)
		{
		    found = 1;
		    ret = ERR;
		    start_day = day;
		    start_hour = hour;
		}
	    }
	    else
	    {
		if (local_time_table [day*HOURS_PER_DAY + hour] != -1)
		{
		    found = 0;
		    fprintf (stderr, "Error: undefined time span in variable %s starting at %s:%d ending at %s:%d\n",
			     time_variables[times_idx], 
			     ge_wday_strings[start_day], start_hour,
			     ge_wday_strings[day], hour);
		    fprintf (logfp, "Error: undefined time span in variable %s starting at %s:%d ending at %s:%d\n",
			     time_variables[times_idx], 
			     ge_wday_strings[start_day], start_hour,
			     ge_wday_strings[day], hour);
		}
	    }
	}
    }
    if (found)
    {
	fprintf (stderr, "Error: undefined time span in variable %s starting at %s:%d ending at %s:%d\n",
		 time_variables[times_idx], 
		 ge_wday_strings[start_day], start_hour,
		 ge_wday_strings[DAYS_PER_WEEK-1], HOURS_PER_DAY);
	fprintf (logfp, "Error: undefined time span in variable %s starting at %s:%d ending at %s:%d\n",
		 time_variables[times_idx], 
		 ge_wday_strings[start_day], start_hour,
		 ge_wday_strings[DAYS_PER_WEEK-1], HOURS_PER_DAY);
    }
    return ret;
}
/*----------------------------------------------------------------------------
 * fill_global_time_table ()			    - fill global time tables
 *----------------------------------------------------------------------------
 */
static int
fill_global_time_table (int times_idx, int start_time, int end_time)
{
    int	i;

    for (i = start_time; i < end_time; i++)
    {
	if (time_table[i] < 0)
	{
	    time_table[i] = times_idx;
	}
	else
	{
	    if (time_table[i] == times_idx)
	    {
		fprintf (stderr,
		 "Error: overlapping time ranges in variable %s\n",
		     time_variables[times_idx]);
		fprintf (logfp,
		 "Error: overlapping Y-time ranges in variable %s\n",
		     time_variables[times_idx]);
	    }
	    else
	    {
		fprintf (stderr,
		 "Error: overlapping time ranges in variables\n       %s and %s\n",
		     time_variables[time_table[i]], time_variables[times_idx]);
		fprintf (logfp,
		 "Error: overlapping time ranges in variables\n       %s and %s\n",
		     time_variables[time_table[i]], time_variables[times_idx]);
	    }
	    return (ERR);
	}
    }

    return (OK);
} /* fill_global_time_table (times_idx, start_time, end_time) */

static int
fill_local_time_table (int times_idx, int start_time, int end_time)
{
    int	i;

    for (i = start_time; i < end_time; i++)
    {
	if (local_time_table[i] < 0)
	{
	    local_time_table[i] = 0;
	}
	else
	{
	    fprintf (stderr, "Error: overlapping time ranges in variable %s\n",
		     time_variables[times_idx]);
	    fprintf (logfp, "Error: overlapping time ranges in variable %s\n",
		     time_variables[times_idx]);
	    return (ERR);
	}
    }
    return (OK);
} /* fill_local_time_table (times_idx, start_time, end_time) */
 
static int
fill_time_table (int times_idx, int lcr, int start_time, int end_time)
{
    int res = OK;
 
    if (lcr)
    {
	res = fill_global_time_table (times_idx, start_time, end_time);
    }
    if (res == OK)
    {
	res = fill_local_time_table (times_idx, start_time, end_time);
    }
    return res;
}


/*----------------------------------------------------------------------------
 *  check_a_time_value (int times_idx,
 *----------------------------------------------------------------------------
 */
static int
check_a_time_value (int times_idx, int lcr,
		    char * start_week_day_str, char * end_week_day_str,
		    char * start_hour_str, char * end_hour_str)
{
    int	    start_day;
    int	    end_day;
    int	    start_hour;
    int	    end_hour;
    int	    day;
    int	    start_time;
    int	    end_time;

    start_day   = convert_week_day_to_day (start_week_day_str);
    end_day	= convert_week_day_to_day (end_week_day_str);

    if (start_day < 0)
    {
	fprintf (stderr, "Error: format error in variable %s: no weekday: %s\n",
		 time_variables[times_idx], start_week_day_str);
	fprintf (logfp, "Error: format error in variable %s: no weekday: %s\n",
		 time_variables[times_idx], start_week_day_str);
	return (ERR);
    }

    if (end_day < 0)
    {
	fprintf (stderr, "Error: format error in variable %s: no weekday: %s\n",
		 time_variables[times_idx], end_week_day_str);
	fprintf (logfp, "Error: format error in variable %s: no weekday: %s\n",
		 time_variables[times_idx], end_week_day_str);
	return (ERR);
    }

    start_hour	= atoi (start_hour_str);
    end_hour	= atoi (end_hour_str);

    if (start_hour < 0 || start_hour >= 24)
    {
	fprintf (stderr,
		 "Error: format error in variable %s: no valid hour: %s\n",
		 time_variables[times_idx], start_hour_str);
	fprintf (logfp,
		 "Error: format error in variable %s: no valid hour: %s\n",
		 time_variables[times_idx], start_hour_str);
	return (ERR);
    }

    if (end_hour <= 0 || end_hour > 24)
    {
	fprintf (stderr,
		 "Error: format error in variable %s: no valid hour: %s\n",
		 time_variables[times_idx], end_hour_str);
	fprintf (logfp,
		 "Error: format error in variable %s: no valid hour: %s\n",
		 time_variables[times_idx], end_hour_str);
	return (ERR);
    }

    day = start_day;

    for (;;)
    {
	if (start_hour > end_hour)
	{
	    start_time  = day * HOURS_PER_DAY + start_hour;
	    end_time    = day * HOURS_PER_DAY + HOURS_PER_DAY;

	    if (fill_time_table (times_idx, lcr, start_time, end_time) != OK)
	    {
		return (ERR);
	    }

	    start_time  = day * HOURS_PER_DAY + 0;
	    end_time    = day * HOURS_PER_DAY + end_hour;

	    if (fill_time_table (times_idx, lcr, start_time, end_time) != OK)
	    {
		return (ERR);
	    }
	}
	else
	{
	    start_time  = day * HOURS_PER_DAY + start_hour;
	    end_time    = day * HOURS_PER_DAY + end_hour;

	    if (fill_time_table (times_idx, lcr, start_time, end_time) != OK)
	    {
		return (ERR);
	    }
	}

	if (day == end_day)
	{
	    break;
	}

	day++;

	if (day == DAYS_PER_WEEK)
	{
	    day = 0;
	}
    }

    return (OK);
} /* check_a_time_value (...) */

/*----------------------------------------------------------------------------
 *  check_time_values (int times_idx)	- check time values of variable
 *
 *  Example:
 *
 *  Mo-Fr:08-18:0.032:Y Mo-Fr:18-08:0.025:Y Sa-Su:00-24:0.025:Y
 *----------------------------------------------------------------------------
 */
static int
check_time_values (int times_idx)
{
    char *  week_range_str;
    char *  hour_range_str;
    char *  yes_no_str;
    char *  start_week_day_str;
    char *  end_week_day_str;
    char *  start_hour_str;
    char *  end_hour_str;
    char *  p;
    char *  pp;
    char *  ppp;

    p = time_values[times_idx];
    init_local_time_table ();

    do
    {
	pp = strchr (p, ' ');

	if (pp)
	{
	    *pp++ = '\0';

	    while (*pp == ' ')
	    {
		pp++;
	    }

	    if (! *pp)
	    {
		pp = (char *) NULL;
	    }
	}

	ppp = strchr (p, ':');

	if (ppp)
	{
	    week_range_str = p;
	    *ppp++ = '\0';
	}
	else
	{
	    fprintf (stderr, "Error: format error in variable %s\n",
		     time_variables[times_idx]);
	    fprintf (logfp, "Error: format error in variable %s\n",
		     time_variables[times_idx]);
	    return (ERR);
	}

	p = ppp;
	ppp = strchr (p, ':');

	if (ppp)
	{
	    hour_range_str = p;
	    *ppp++ = '\0';
	}
	else
	{
	    fprintf (stderr, "Error: format error in variable %s\n",
		     time_variables[times_idx]);
	    fprintf (logfp, "Error: format error in variable %s\n",
		     time_variables[times_idx]);
	    return (ERR);
	}

	p = ppp;
	ppp = strchr (p, ':');

	if (ppp)
	{
	    *ppp++ = '\0';
	}
	else
	{
	    fprintf (stderr, "Error: format error in variable %s\n",
		     time_variables[times_idx]);
	    fprintf (logfp, "Error: format error in variable %s\n",
		     time_variables[times_idx]);
	    return (ERR);
	}

	yes_no_str = ppp;

	if (strlen (week_range_str) != 5 ||
	    *(week_range_str + 2)   != '-' ||
	    strlen (hour_range_str) != 5 ||
	    *(hour_range_str + 2)   != '-')
	{
	    fprintf (stderr, "Error: format error in variable %s\n",
		     time_variables[times_idx]);
	    fprintf (logfp, "Error: format error in variable %s\n",
		     time_variables[times_idx]);
	    return (ERR);
	}

	if (! (strcmp (yes_no_str, "Y") && strcmp (yes_no_str, "y") &&
	       strcmp (yes_no_str, "N") && strcmp (yes_no_str, "n")))
	{
	    int lcr = ! (strcmp (yes_no_str, "Y") && strcmp (yes_no_str, "y"));
	    *(week_range_str + 2)   = '\0';
	    start_week_day_str	    = week_range_str;
	    end_week_day_str	    = week_range_str + 3;

	    *(hour_range_str + 2)   = '\0';
	    start_hour_str	    = hour_range_str;
	    end_hour_str	    = hour_range_str + 3;

	    if (check_a_time_value (times_idx, lcr,
				start_week_day_str, end_week_day_str,
				start_hour_str, end_hour_str) != OK)
	    {
		return (ERR);
	    }
	}
	else
	{
	    fprintf (stderr, "Error: format error in variable %s\n",
		     time_variables[times_idx]);
	    fprintf (logfp, "Error: format error in variable %s\n",
		     time_variables[times_idx]);
	    return (ERR);
	}

	p = pp;
    } while (pp);

    return check_local_time_table (times_idx);
} /* check_time_values (times_idx) */


/*----------------------------------------------------------------------------
 *  check_time_variables (void)
 *----------------------------------------------------------------------------
 */
static int
check_time_variables (void)
{
    char    varname[64];
    char *  p;
    int	    n_time_variables	= 0;
    int	    pppoe_used		= FALSE;
    int	    is_pptp		= FALSE;
    int	    n_isdn_circuits	= 0;
    int	    times_idx;
    int	    i;
    int	    j;
    int	    ret = OK;

    p = get_variable ("OPT_PPPOE");

    if (p && ! strcmp (p, "yes"))
    {
	pppoe_used = TRUE;
	n_time_variables++;
    }

    p = get_variable ("OPT_PPTP");

    if (p && ! strcmp (p, "yes"))
    {
	pppoe_used  = TRUE;
	is_pptp	    = TRUE;
	n_time_variables++;
    }

    p = get_variable ("OPT_ISDN");

    if (p && ! strcmp (p, "yes"))
    {
	n_isdn_circuits = atoi (get_variable ("ISDN_CIRCUITS_N"));
	n_time_variables += n_isdn_circuits;
    }

    if (n_time_variables == 0)
    {
	return (OK);
    }

    time_values	    = (char **) malloc (n_time_variables * sizeof (char *));
    time_variables  = (char **) malloc (n_time_variables * sizeof (char *));

    for (i = 0; i < n_isdn_circuits; i++)
    {
	sprintf (varname, "ISDN_CIRC_%d_TIMES", i + 1);
	time_variables[i] = strsave (varname);
	p = get_variable (varname);
	time_values[i] = strsave (p ? p : "");
    }

    if (pppoe_used == TRUE)
    {
	if (is_pptp == TRUE)
	{
	    time_variables[i] = strsave ("PPTP_TIMES");
	    p = get_variable ("PPTP_TIMES");
	    time_values[i] = strsave (p ? p : "");
	}
	else
	{
	    time_variables[i] = strsave ("PPPOE_TIMES");
	    p = get_variable ("PPPOE_TIMES");
	    time_values[i] = strsave (p ? p : "");
	}
    }

    for (i = 0; i < DAYS_PER_WEEK; i++)
    {
	for (j = 0; j < HOURS_PER_DAY; j++)
	{
	    time_table[i * HOURS_PER_DAY + j] = -1;
	}
    }

    for (times_idx = 0; times_idx < n_time_variables; times_idx++)
    {
	if (times_idx < n_isdn_circuits)
	{
	    sprintf (varname, "ISDN_CIRC_%d_ROUTE", times_idx + 1);
	    p = get_variable (varname);

	    if (p && strcmp (p, "default") != 0 && strcmp (p, "0.0.0.0") != 0)
	    {
		continue;			    /* no lcr circuit, skip */
	    }
	}

	if (check_time_values (times_idx) != OK)
	{
	    ret = (ERR);
	}
    }

    return ret;
} /* check_time_variables () */

/*----------------------------------------------------------------------------
 *  check_all_variables (void)
 *----------------------------------------------------------------------------
 */
int
check_all_variables (void)
{
    char *  name;
    char *  opt_var;
    char *  var_n;
    char *  opt_var_content;
    char *  opt_isdn;
    char *  opt_pppoe;
    char *  opt_pptp;
    char *  start_imond;
    char *  content;
    int	    isdn_used;
    int	    pppoe_used;
    int	    imond_used;
    int	    value;
    int	    i;

    opt_isdn	= get_variable ("OPT_ISDN");
    opt_pppoe	= get_variable ("OPT_PPPOE");
    opt_pptp	= get_variable ("OPT_PPTP");

    if (opt_isdn && ! strcmp (opt_isdn, "yes"))
    {
	isdn_used = TRUE;
    }
    else
    {
	isdn_used = FALSE;
    }

    if (opt_pppoe && ! strcmp (opt_pppoe, "yes"))
    {
	pppoe_used = TRUE;
    }
    else if (opt_pptp && ! strcmp (opt_pptp, "yes"))
    {
	pppoe_used = TRUE;
    }
    else
    {
	pppoe_used = FALSE;
    }

    start_imond = get_variable ("START_IMOND");

    if (start_imond && ! strcmp (start_imond, "yes"))
    {
	imond_used = TRUE;
    }
    else
    {
	imond_used = FALSE;
    }

    for (i = 0; i < c_variables_used; i++)
    {
	name	= c_variables[i].name;
	value	= c_variables[i].value;
	opt_var = c_variables[i].opt_var;

	if (opt_var)
	{
	    opt_var_content = get_variable (opt_var);

	    if (! opt_var_content)
	    {
		fprintf (stderr,
		"variable %s is dependent from %s, which is not defined\n",
			 name, opt_var);
		fprintf (logfp,
		"variable %s is dependent from %s, which is not defined\n",
			 name, opt_var);
		return (ERR);
	    }

	    if (! strcmp (opt_var_content, "no"))
	    {
		content = get_variable (name);

		if (! strncmp (name, "OPT_", 4) &&  /* optvar dependent ?   */
		    content && ! strcmp (content, "yes"))
		{
		    printf ("Warning: %s='%s' ignored, because %s='no'\n",
			    name, content, opt_var);

		    set_variable_content (name, "no");
		}
		continue;
	    }
	}

	var_n = c_variables[i].var_n;

	if (var_n)
	{
	    if (chkvar_idx (name, var_n, value) != OK)
	    {
		return (ERR);
	    }
	}
	else
	{
	    if (chkvar (name, value) != OK)
	    {
		return (ERR);
	    }
	}
    }

    if (isdn_used || pppoe_used)
    {
	if (! imond_used)
	{
	    fprintf (stderr,
		     "Error: if you use ISDN and/or PPPOE/PPTP, you must set START_IMOND='yes'\n");
	    fprintf (logfp,
		     "Error: if you use ISDN and/or PPPOE/PPTP, you must set START_IMOND='yes'\n");
	    return (ERR);
	}
    }

    if (! isdn_used && ! pppoe_used)
    {
	if (imond_used)
	{
	    fprintf (stderr,
		     "Error: if you don't use ISDN or PPPOE/PPTP, you must set START_IMOND='no'\n");
	    fprintf (logfp,
		     "Error: if you don't use ISDN or PPPOE/PPTP, you must set START_IMOND='no'\n");
	    return (ERR);
	}
    }

    if (check_time_variables () != OK)
    {
	return (ERR);
    }

    return (OK);
} /* check_all_variables () */
